
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qcombobox.h>

#include "CorrectionBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Utilities.h"


CorrectionBox::CorrectionBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("CorrectionBox")
{
   QHGroupBox   *ModeBox;
   QBoxLayout   *l_Layout;
   int           i;

   TRACE << "Correction Started\n" ;

   setCaption("Color Correction");

   m_pc_GlobalStorage = pc_Global;
   m_b_Visible        = false;
   m_i_CurrentMode    = false;

   // create a vertical layout
   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   // block for the mode
   {

      m_ModeWidget = new QButtonGroup(3,
                                    QGroupBox::Horizontal,
                                    "Mode",
                                    m_w_TopWidget);
      l_Layout->addWidget(m_ModeWidget);

      m_ModeLightnessButton  = new QRadioButton("Lighness",m_ModeWidget);
      m_ModeColorButton      = new QRadioButton("Color",m_ModeWidget);

      m_ModeLightnessButton->setChecked(1);
      connect( m_ModeWidget,
              SIGNAL(clicked(int)),
              SLOT(ModeChanged(int)) );
   }


   // box for lighness
   m_LightnessBox = new QVGroupBox("Lighness",m_w_TopWidget);
   l_Layout->addWidget(m_LightnessBox);
   {
      m_pc_LighnessCorrection = new QUCorrectionCurve(m_LightnessBox,
                                                      m_pc_GlobalStorage);

      connect(m_pc_LighnessCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(LightnessCurveChanged(unsigned char *)) );
   } // box for lightness


   // box for Color
   m_ColorBox = new QVGroupBox("Color",m_w_TopWidget);
   l_Layout->addWidget(m_ColorBox);
   {
      m_pc_RedCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0xff,0,0) );
      m_pc_GreenCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0,0xff,0));
      m_pc_BlueCorrection = new QUCorrectionCurve(m_ColorBox,
                                                 m_pc_GlobalStorage,
                                                 QUCorrectionCurve::OptionSmall,
                                                 PIXEL_MAKE(0,0,0xff));

      connect(m_pc_RedCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(RedCurveChanged(unsigned char *)) );
      connect(m_pc_GreenCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(GreenCurveChanged(unsigned char *)) );
      connect(m_pc_BlueCorrection,
              SIGNAL(CurveChanged(unsigned char *)),
              SLOT(BlueCurveChanged(unsigned char *)) );
   }

   // box for the mode
   ModeBox = new QHGroupBox("Mode",m_w_TopWidget);
   l_Layout->addWidget(ModeBox);
   {
      m_CorrectionVisibleCheck = new QCheckBox("Visible", ModeBox);

      m_ApplyButton = new QPushButton("Apply",  ModeBox);


      connect(m_CorrectionVisibleCheck,
              SIGNAL(toggled(bool)),
              SLOT( VisibleChecked(bool) ) );
      connect(m_ApplyButton,
              SIGNAL(clicked() ),
              SLOT(ApplyPressed()) );
   }

   SetSensitivity();

   // add message receiver for disable mode messages
   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction) OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageNewBuffer");

   m_pc_GlobalStorage->m_pc_MessageServer->AddMessageReceiver((MessageCallbackFunction)OnMessageReceiveCB,
                                                       (void *) this,
                                                       "ImageResize");
   // initital lightness in on, color is off
   m_LightnessBox->show();
   m_ColorBox    ->hide();

   // set a 1 - to 1 conversion
   for(i=0;i<256;i++)
   {
      m_uca_CorrectionRed      [i] = (unsigned char) i;
      m_uca_CorrectionGreen    [i] = (unsigned char) i;
      m_uca_CorrectionBlue     [i] = (unsigned char) i;
      m_uca_CorrectionLightness[i] = (unsigned char) i;

   }
} // CorrectionBox()

//===========================================================================
//
//===========================================================================

CorrectionBox::~CorrectionBox()
{

} // ~CorrectionBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void CorrectionBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
   TRACE << "Message Received member this = " << (int) this << "\n";

   if(qs_MessageType.compare("ImageResize") ==0)
   {
      Disable();
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
      Disable();
   }
} // OnMessageReceive()

//===========================================================================
//
//===========================================================================


void CorrectionBox::SetSensitivity()
{
} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void CorrectionBox::LightnessCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Lighness Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionLightness,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // LightnessCurveChanged()

//===========================================================================
//
//===========================================================================

void CorrectionBox::RedCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Red Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionRed,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // RedCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::GreenCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Green Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionGreen,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // GreenCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::BlueCurveChanged(unsigned char *uc_CorrectionCurve)
{
   TRACE << "Blue Curve Changed\n" ;

   // copy the curve also into a local buffer
   memcpy(m_uca_CorrectionBlue,uc_CorrectionCurve,256);

   // copy the correction curve into the zoom object
   m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   // force a redraw
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   }

} // BlueCurveChanged()
//===========================================================================
//
//===========================================================================

void CorrectionBox::ApplyPressed()
{
   TRACE << "Apply pressed\n" ;

   // apply the correction to the image
   if(m_i_CurrentMode == 0)
   {
      m_pc_GlobalStorage->m_pc_MainImage->ConvertColors(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   }
   else if(m_i_CurrentMode == 1)
   {
      m_pc_GlobalStorage->m_pc_MainImage->ConvertColors(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   }


   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   m_CorrectionVisibleCheck->setChecked(0);

   // tell that the image is changed
   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                 this,
                                                 "ImageChanged");

} // ApplyPressed()

//===========================================================================
//
//===========================================================================

void CorrectionBox::VisibleChecked(bool b_On)
{
   m_b_Visible = b_On;

   // tell state to zoom
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(true);
   }
   else
   {
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(false);
   }
   // force a redraw
   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
} // VisibleChecked()

//===========================================================================
//
//===========================================================================

void CorrectionBox::Disable()
{
   if(m_b_Visible)
   {
      // disable the button
      m_CorrectionVisibleCheck->setChecked(0);
      // disable it in the zoom
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionEnabled(false);

      m_b_Visible = false;
      // force a redraw
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   } // if was visible

} // Disable

//===========================================================================
//
//===========================================================================

void CorrectionBox::ModeChanged(int i_NewMode)
{
   m_i_CurrentMode = i_NewMode;

   // switch from color to lightness and vice versa
   if(m_i_CurrentMode ==0)
   {
      // to lightness
      m_LightnessBox->show();
      m_ColorBox    ->hide();
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness,
                           m_uca_CorrectionLightness);
   }
   else if (m_i_CurrentMode ==1)
   {
      // to color
      m_LightnessBox->hide();
      m_ColorBox    ->show();
      m_pc_GlobalStorage->m_pc_Zoom->SetCorrectionCurve(m_uca_CorrectionRed,
                           m_uca_CorrectionGreen,
                           m_uca_CorrectionBlue);
   }

   // let the widget resize to the new layout
   ResizeDisplay();

   // apply the new curves in the image
   if(m_b_Visible)
   {
      m_pc_GlobalStorage->m_pc_ImageFrame->RedrawAll();
   } // if visible


} // ModeChanged()

//===========================================================================
//
//===========================================================================

