
#include <stdio.h>

#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qfont.h>
#include <qfontmetrics.h>


#include "GraphikBox.h"

#include "PixelBuffer.h"
#include "QtUtilities.h"
#include "ImageFrame.h"
#include "MessageServer.h"
#include "Selection.h"
#include "BufferPainter.h"

#include "QULabelSlider.h"
#include "MaskSettings.h"


GraphikBox::GraphikBox(QWidget *parent,
                        GlobalStorage *pc_Global) :
               	StandardDialog(parent,pc_Global),
                Tracer("GraphikBox")
{
   QBoxLayout   *l_Layout;

   QHGroupBox   *ActivationBox;
   QVGroupBox   *LineBox;
   QVGroupBox   *TransparencyBox;

   m_pc_GlobalStorage = pc_Global;

   setCaption("Draw Graphic");
   
   m_i_LineWidth =1;


   m_pc_BufferPainter = new BufferPainter();
   m_pc_BufferPainter->SetLineWidth(m_i_LineWidth);
   m_pc_BufferPainter->SetDrawInnerEnable(false);
   m_pc_BufferPainter->SetTransparencyEnable(false);
   m_pc_BufferPainter->SetTransparencyFactor(50);



   l_Layout = new  QVBoxLayout( m_w_TopWidget,1);


   // -- box with the activation buttons
   ActivationBox = new QHGroupBox("Graphics",m_w_TopWidget);
   ActivationBox->setColumns(2);
   l_Layout->addWidget(ActivationBox);
   {
      // buttons to establish painting mode

      m_PointButton     = new QUPixelButton(ActivationBox,
                                            "GraphPoint",
                                            "Point");
      m_LineButton      = new QUPixelButton(ActivationBox,
                                            "GraphLine",
                                            "Line");
      m_QuadratButton   = new QUPixelButton(ActivationBox,
                                            "GraphQuadrat",
                                             "Quadrat");
      m_RectangleButton = new QUPixelButton(ActivationBox,
                                            "GraphRectangle",
                                            "Rectangle");
      m_CircleButton    = new QUPixelButton(ActivationBox,
                                            "GraphCircle",
                                            "Circle");
      m_EllipseButton   = new QUPixelButton(ActivationBox,
                                            "GraphEllipse",
                                            "Ellipse");
      m_FreeDrawButton   = new QUPixelButton(ActivationBox,
                                             "GraphFree",
                                             "Free Hand"
                                             );

      connect( m_PointButton ,
               SIGNAL(clicked()),
               SLOT( ActivatePoint() ) );
      connect( m_LineButton,
               SIGNAL(clicked()),
               SLOT( ActivateLine()) );
      connect( m_QuadratButton,
               SIGNAL(clicked()),
               SLOT( ActivateQuadrat()) );
      connect( m_RectangleButton,
               SIGNAL(clicked()),
               SLOT( ActivateRectangle()) );
      connect( m_EllipseButton,
               SIGNAL(clicked()),
               SLOT( ActivateEllipse() ) );
      connect( m_CircleButton, 
               SIGNAL(clicked()),
               SLOT( ActivateCircle() ) );
      connect( m_FreeDrawButton, 
               SIGNAL(clicked()), 
               SLOT( ActivateFreeDraw() ) );
   }

   // box with the line styles
   LineBox = new QVGroupBox("Line Style",m_w_TopWidget);
   l_Layout->addWidget(LineBox);

   {
      QULabelSlider *pc_NewSlider;
      pc_NewSlider = new QULabelSlider(LineBox,true,
                                       0,30,
                                       m_i_LineWidth,
                                       "Line Width",6);

      m_LineColorWidget = new QUColorOrPattern(LineBox,
                                               m_pc_GlobalStorage
                                               );
      m_LineColorWidget->SetColor(0x00ff00);
      

      connect( pc_NewSlider,
               SIGNAL(valueChanged(int)),
               SLOT( LineThicknessChanged(int )) );

      connect( m_LineColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( LineColorChanged(Pixel ) )
             );
      connect( m_LineColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( LinePatternChanged(PixelBuffer * ) )
             );
      connect( m_LineColorWidget,
               SIGNAL(ModeChange(bool)) ,
               SLOT( LineModeChanged(bool ) )
             );

   } // box for line style

   // -- box for the inner style
   InnerBox = new QVGroupBox("Inner Style",m_w_TopWidget);
   l_Layout->addWidget(InnerBox);

   {
      // inner is off
      m_DrawInnerEnable = new QCheckBox("Draw Inner",InnerBox);

      m_InnerColorWidget = new QUColorOrPattern(InnerBox,
                                               m_pc_GlobalStorage);
      m_InnerColorWidget->SetColor(0x044ff);

      connect( m_DrawInnerEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( DrawInnerChanged(bool) )
             );

      connect( m_InnerColorWidget,
               SIGNAL(ColorChanged(Pixel)) ,
               SLOT( InnerColorChanged(Pixel ) )
             );
      connect( m_InnerColorWidget,
               SIGNAL(ImageDropped(PixelBuffer *)) ,
               SLOT( InnerPatternChanged(PixelBuffer * ) )
             );
      connect( m_InnerColorWidget,
               SIGNAL(ModeChange(bool)) ,
               SLOT( InnerModeChanged(bool ) )
             );

   } // box for inner settings

   // box for transparency draw mode
   TransparencyBox = new QVGroupBox("Transparency",m_w_TopWidget);
   l_Layout->addWidget(TransparencyBox);

   {
      // inner is off
      m_TransparencyEnable = new QCheckBox("Enable",TransparencyBox);

      m_TransparencySlider = new QULabelSlider(TransparencyBox,true,
                                       0,100,50,
                                       "Factor",6);

      connect( m_TransparencyEnable,
               SIGNAL(toggled(bool)) ,
               SLOT( TransparencyEnableChanged(bool) )
             );
      connect( m_TransparencySlider,
               SIGNAL(valueChanged(int)),
               SLOT( TransparencyFactorChanged(int )) );

   } // block for tranparency box

   // final settings
   m_pc_BufferPainter->SetBorderColor(m_LineColorWidget->GetColor());
   m_pc_BufferPainter->SetInnerColor(m_InnerColorWidget->GetColor());
   
   InnerBox->hide();

} // GraphikBox()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void GraphikBox::ActivatePoint()
{
   InnerBox->hide();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionPoint,
                                 "Point",
                                 Callback(SelectionEventArrivedCB,this) );

   m_i_CurrentGraphikMode = GraphikPoint;
} // ActivatePoint()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateLine()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionLine,
                                                  "Line",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikLine;

} // ActivateLine()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateRectangle()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionRectangle,
                                                  "Rectangle",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikRectangle;

} // ActivateRectangle()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateQuadrat()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionQuadrat,
                                                  "Quadrat",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikQuadrat;

} // ActivateQuadrat()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateEllipse()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionEllipse,
                                                  "Ellipse",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikEllipse;

} // ActivateEllipse()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void GraphikBox::ActivateCircle()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionCircle,
                                                  "Circle",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikCircle;

} // ActivateCircle()

//===========================================================================
//
//===========================================================================

void GraphikBox::ActivateFreeDraw()
{
   InnerBox->show();
   ResizeDisplay();
   m_pc_GlobalStorage->m_pc_Selection->ActivateSelection(Selection::SelectionFreeDraw,
                                                  "Free",
                                                  Callback(SelectionEventArrivedCB,this));

   m_i_CurrentGraphikMode = GraphikFreeDraw;

} // ActivateCircle()

//===========================================================================
//
//===========================================================================

void GraphikBox::OnMessageReceive(QString qs_MessageType,
                                 void *p_Data)
{
//fprintf(stderr,"Message Received member this = %x\n",this);
   if(qs_MessageType.compare("ImageResize") ==0)
   {
   }
   if(qs_MessageType.compare("ImageNewBuffer") ==0)
   {
   }
} // OnMessageReceive()


//===========================================================================
//
//===========================================================================


void *GraphikBox::SelectionEventArrived(void *pc_Data)
{
//fprintf(stderr,"SelectionEventArrived\n");
   SelectionCallbackData *cb;
   cb = (SelectionCallbackData *) pc_Data;
   if(cb->i_CallbackReason == SelectionCallbackData:: ReasonOperationDisabled)
   {
      // clean up unused structures
   }
   else if(cb->i_CallbackReason == SelectionCallbackData:: ReasonSelection)
   {

      switch(m_i_CurrentGraphikMode)
      {
      case GraphikPoint:
         DrawPoint();
         break;
      case GraphikLine:
         DrawLine();
         break;
      case GraphikRectangle:
         DrawRectangle();
         break;
      case GraphikQuadrat:
         DrawQuadrat();
         break;
      case GraphikEllipse:
         DrawEllipse();
         break;
      case GraphikCircle:
         DrawCircle();
         break;
      case GraphikFreeDraw:
         DrawFreeDraw();
         break;
      default:
         break;
      }
   } // else reasonselection

   return NULL;
} // SelectionEventArrived()


//===========================================================================
//
//===========================================================================


void *GraphikBox::SelectionEventArrivedCB(void *Thisdata,void *pc_Data)
{
   GraphikBox *pc_This = (GraphikBox *) Thisdata;

   return pc_This->SelectionEventArrived(pc_Data);

} // SelectionEventArrivedCB()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawPoint()
{
   RectRegion c_ChangedRegion;
   int        i_X;
   int        i_Y;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionPoint(i_X,i_Y);


   m_pc_BufferPainter->DrawPoint(GetDestinationBuffer(),i_X,i_Y);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");

} // DrawPoint()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawLine()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionLine(i_X1,i_Y1,i_X2,i_Y2);


   if(   i_X1 == i_X2
      && i_Y1 == i_Y2
     )
   {
      // start end end are the same point. forget the line
      return;
   }


   m_pc_BufferPainter->DrawLine(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);


   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawLine()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawRectangle()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionRectangle(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawRectangle(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawRectangle()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawQuadrat()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionQuadrat(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawRectangle(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawQuadrat()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawEllipse()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_X2;
   int        i_Y2;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionEllipse(i_X1,i_Y1,i_X2,i_Y2);

   m_pc_BufferPainter->DrawEllipse(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_X2,i_Y2);



   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawEllipse()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawCircle()
{
   RectRegion c_ChangedRegion;
   int        i_X1;
   int        i_Y1;
   int        i_Radius;

   m_pc_GlobalStorage->m_pc_Selection->GetSelectionCircle(i_X1,i_Y1,i_Radius);

   m_pc_BufferPainter->DrawEllipse(GetDestinationBuffer(),
             i_X1,i_Y1,
             i_Radius, i_Radius);



   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();


   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawCircle()

//===========================================================================
//
//===========================================================================


void GraphikBox::DrawFreeDraw()
{
   QValueVector<PixelCoordinate> *pc_List;
   RectRegion      c_ChangedRegion;
   int i_Size;

   pc_List = m_pc_GlobalStorage->m_pc_Selection->GetSelectionPointList();
   if(pc_List == NULL)
   {
      return;
   }
   i_Size = pc_List->size();
//fprintf(stderr,"Size = %d\n",i_Size);
   if(i_Size ==0)
   {
      return;
   }
   m_pc_BufferPainter->DrawFreeFigure(GetDestinationBuffer(),
                       pc_List);

   c_ChangedRegion = m_pc_BufferPainter->GetChangedRegion();

   m_pc_GlobalStorage->m_pc_ImageFrame->RedrawBufferRegion(c_ChangedRegion);

   m_pc_GlobalStorage->m_pc_MessageServer->SendMessage(0,
                                                       this,
                                                       "ImageChanged");
} // DrawFreeDraw()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineThicknessChanged(int i_Width)
{
   m_pc_BufferPainter->SetLineWidth(i_Width);
   m_i_LineWidth = i_Width;

} // LineThicknessChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::DrawInnerChanged(bool b_on)
{
   m_pc_BufferPainter->SetDrawInnerEnable(b_on);
}  // DrawInnerChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::TransparencyEnableChanged(bool b_on)
{
   m_pc_BufferPainter->SetTransparencyEnable(b_on);
} // ransparencyEnableChanged

//===========================================================================
//
//===========================================================================

void GraphikBox::TransparencyFactorChanged(int i_Factor)
{
   m_pc_BufferPainter->SetTransparencyFactor(i_Factor);
} // TransparencyFactorChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetBorderColor(c_Color);
} // LineColorChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerColorChanged(Pixel c_Color)
{
   m_pc_BufferPainter->SetInnerColor(c_Color);
} // InnerColorChanged()

//===========================================================================
//
//===========================================================================

PixelBuffer *GraphikBox::GetDestinationBuffer()
{
   PixelBuffer *pc_Destination;
   
   pc_Destination = m_pc_GlobalStorage->m_pc_MainImage;
   if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSafety)
   {
      pc_Destination = pc_Destination->GetSafetyMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else if( m_pc_GlobalStorage->m_pc_MaskSettings->GetPaintMode() == MaskSettings::MaskPaintModeInSelection)
   {
      pc_Destination = pc_Destination->GetSelectionMask();
      if(m_pc_GlobalStorage->m_pc_MaskSettings->GetDraw() == MaskSettings::MaskPaintDrawExtend)
      {
         m_pc_BufferPainter->SetMaskMode(true,true);
      }
      else
      {
         m_pc_BufferPainter->SetMaskMode(true,false);
      }
   }
   else
   {
      m_pc_BufferPainter->SetMaskMode(false,false);
   }

   return pc_Destination;
} // GetDestinationBuffer()

//===========================================================================
//
//===========================================================================

void GraphikBox::LinePatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetBorderPattern(pc_Buffer);
} // LinePatternChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::LineModeChanged(bool b_PatternMode)
{
   m_pc_BufferPainter->SetBorderDrawMode(b_PatternMode);
}  // LineModeChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerPatternChanged(PixelBuffer *pc_Buffer)
{
   m_pc_BufferPainter->SetInnerPattern(pc_Buffer);
} // InnerPatternChanged()

//===========================================================================
//
//===========================================================================

void GraphikBox::InnerModeChanged(bool b_PatternMode)
{
   m_pc_BufferPainter->SetInnerDrawMode(b_PatternMode);
}  // InnerModeChanged()

//===========================================================================
//
//===========================================================================


