
#include <math.h>

#include <qgrid.h>
#include <qlayout.h>

#include "QUCorrectionCurve.h"
#include "GraphicObjectMain.h"
#include "GraphicObject.h"
#include "Tracer.h"
#include <qvgroupbox.h>
#include <qhgroupbox.h>

#include "QtUtilities.h"

#include "ApplicationStorage.h"

//===========================================================================
//
//===========================================================================

QUCorrectionCurve::QUCorrectionCurve(QWidget *parent, 
                                     GlobalStorage *pc_ParentGlobalStorage,
                                     int i_Options,
                                     Pixel p_Mask)
                        : QWidget(parent), 
                          Tracer("QUCorrectionCurve")
{
   QHBoxLayout  *MainWidgetLayout;

   QGridLayout *grid;
   QWidget     *LeftWidget;
   QWidget     *RightWidget;

   int          i_CharWidth;
   int          i_X;
   int          i_Y;
   int          i;

   // check whether regular or small mode is selected
   m_b_Small = false;
   if(i_Options & OptionSmall)
   {
      m_b_Small =true;
   }

   //remember the global storage
   m_pc_ParentGlobalStorage = pc_ParentGlobalStorage;

   m_i_CurveMode         = CurveModeLinear;
   m_i_CurrentCurvePoints= 0;
   m_f_GammaValue        = 1.0;
   m_b_SelfUpdate        = false;
   p_ColorMask           = p_Mask;

   i_CharWidth = g_Application->m_pc_QtUtility->GetMaximalFontWidth();

   // make a left widget for buttons, right for graphik
   MainWidgetLayout  = new  QHBoxLayout(this,2);
   LeftWidget        = new QWidget(this);
   RightWidget       = new QWidget(this);
   MainWidgetLayout->addWidget(LeftWidget);
   MainWidgetLayout->addWidget(RightWidget);

   // block for the action Widget;
   {
      QVBoxLayout  *ActionLayout;
      ActionLayout  = new  QVBoxLayout(LeftWidget,2);

      m_ModeCombo    = new QComboBox(false,LeftWidget);
      m_ModeCombo->insertItem("Linear Lines",CurveModeLinear);
      m_ModeCombo->insertItem("Gamma Curve" ,CurveModeGamma);
      m_ModeCombo->setCurrentItem(m_i_CurveMode);
      ActionLayout->addWidget(m_ModeCombo);

      m_AddButton    = new QPushButton("Add Point",LeftWidget);
      ActionLayout->addWidget(m_AddButton);
      m_DeleteButton = new QPushButton("Delete Point",LeftWidget);
      ActionLayout->addWidget(m_DeleteButton);

      m_GammaFactorEdit = new QLineEdit(LeftWidget);
      m_GammaFactorEdit->setMaximumWidth(i_CharWidth*6);
      ActionLayout->addWidget(m_GammaFactorEdit);
      m_ResetButton    = new QPushButton("Reset",LeftWidget);
      ActionLayout->addWidget(m_ResetButton);

      // add a spacer to eat all vertical space
      ActionLayout->addWidget(new QtUtilSpacer(false,true,LeftWidget));

      connect( m_ModeCombo,
               SIGNAL(activated(int)),
               SLOT( LineModeChanged(int )) );

      connect( m_AddButton ,
               SIGNAL(clicked()),
               SLOT( AddPressed() ) );
      connect( m_DeleteButton ,
               SIGNAL(clicked()),
               SLOT( DeletePressed() ) );
      connect( m_ResetButton ,
               SIGNAL(clicked()),
               SLOT( ResetPressed() ) );

      connect( m_GammaFactorEdit,
            SIGNAL(textChanged ( const QString & )),
            SLOT( TextChanged(const QString &) ) );

   } // block for action Widget

   // initial an 1 to 1 conversion
   for(i=0;i<256;i++)
   {
      m_uca_CorrectionCurve[i] = i;
   }

   // small are is 128 , large 256 pixels
   if(m_b_Small)
   {
      m_i_AreaSize = 128;
      m_i_ViewSize = 15;
   }
   else
   {
      m_i_AreaSize = 256;
      m_i_ViewSize = 20;
   }

   // create a grid for the 3 drawing areay
   grid = new QGridLayout( RightWidget, 2, 2,1);

   m_pc_CurveFrame = new ImageFrame(RightWidget,NULL,NULL,
                               ImageFrame::FixedSize ,
                               m_i_AreaSize,
                               m_i_AreaSize);

   m_pc_OriginalFrame = new ImageFrame(RightWidget,NULL,NULL,
                               ImageFrame::FixedSize | ImageFrame::NoSelection,
                               m_i_AreaSize,
                               m_i_ViewSize);

   m_pc_CorrectedFrame = new ImageFrame(RightWidget,NULL,NULL,
                               ImageFrame::FixedSize | ImageFrame::NoSelection,
                               m_i_ViewSize,
                               m_i_AreaSize);

   grid->addWidget(m_pc_CurveFrame,0,0);
   grid->addWidget(m_pc_CorrectedFrame,0,1);
   grid->addWidget(m_pc_OriginalFrame,1,0);


   // creeate pixelbuffer for the main working area
   m_pc_CurveBuffer     = new PixelBuffer(m_i_AreaSize,m_i_AreaSize,BITNUMBER_LONG);
   CleanMainBuffer();
   m_pc_CurveFrame->ReplacePixelbuffer(m_pc_CurveBuffer);

   // create pixelbuffer for the greyscale before transformation
   m_pc_OriginalBuffer  = new PixelBuffer(m_i_AreaSize,m_i_ViewSize,BITNUMBER_LONG);
   
   // fill that buffer
   for(i_X=0;i_X<m_i_AreaSize;i_X ++)
   {
      Pixel p_Pixel;
      int   i_Value;
      
      i_Value = i_X;
      if(m_b_Small)
      {
         i_Value *= 2;
      }
      p_Pixel = PIXEL_MAKE(i_Value,i_Value,i_Value);
      p_Pixel &= p_ColorMask;
      for(i_Y=0;i_Y<m_i_ViewSize;i_Y ++)
      {
         m_pc_OriginalBuffer->Set(i_X,i_Y,p_Pixel);
      }
   }
   m_pc_OriginalFrame->ReplacePixelbuffer(m_pc_OriginalBuffer);

   // create a pixelbuffer for the result
   m_pc_CorrectedBuffer = new PixelBuffer(m_i_ViewSize,m_i_AreaSize,BITNUMBER_LONG);
   CalculateCorrectedBuffer();
   m_pc_CorrectedFrame->ReplacePixelbuffer(m_pc_CorrectedBuffer);

   // get global storage for the main image
   m_pc_CurveGlobalStorage = m_pc_CurveFrame->GetGlobalStorage();

   // create the object-layer for the curve Widget
   m_pc_CurveGlobalStorage->m_pc_ObjectMain->SetVisible(true);

   m_pc_CurveGlobalStorage->m_pc_ObjectMain->CreateLayer("Curve");

   // get layer for the objects
   pc_ObjectLayer = m_pc_CurveGlobalStorage->m_pc_ObjectMain->GetActiveInputLayer();

   // Allocate all points and lines
   m_pc_GammaCurvePoint = new GraphicObjectPoint(m_i_AreaSize / 2.0,
                                                 m_i_AreaSize / 2.0);
   // points are all created, but invisible
   m_pc_GammaCurvePoint->SetVisible(false);
   pc_ObjectLayer->AddObject(m_pc_GammaCurvePoint);

   for(i=0;i<CURVE_MAXIMUM_POINTS;i++)
   {
      m_pc_CurvePoints[i] = new GraphicObjectPoint((float)i,(float)i);
      m_pc_CurvePoints[i]->SetVisible(false);
      pc_ObjectLayer->AddObject(m_pc_CurvePoints[i]);
   }
   
   // same for lines. create them, but let them be invisible
   for(i=0;
       i<(CURVE_MAXIMUM_POINTS+1);
       i++)
   {
      m_pc_CurveLines[i] = new GraphicObjectLine((float)i,(float)i,(float)i+1,(float)i+1);
      m_pc_CurveLines[i]->SetVisible(false);

      // lines should never get selected
      m_pc_CurveLines[i]->SetSelectable(false);
      pc_ObjectLayer->AddObject(m_pc_CurveLines[i]);
   }

   // set only valid input controls active
   SetSensitivity();

   // set the callback for graphic object changes
   m_pc_CurveGlobalStorage->m_pc_ObjectMain->SetEventCallback(
                              Callback(GraphicObjectCallbackCB,this));

   // enable the graphic obejcts
   m_pc_CurveGlobalStorage->m_pc_ObjectMain->ActivateInput();
} // QUCorrectionCurve()

//===========================================================================
//
//===========================================================================

QUCorrectionCurve::~QUCorrectionCurve()
{

} // ~QUCorrectionCurve()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::CalculateCorrectedBuffer()
{
   int i_X;
   int i_Y;
   int i_Value;

   for(i_Y=0;i_Y<m_i_AreaSize;i_Y ++)
   {
      unsigned char uc_NewValue;
      Pixel         p_Pixel;

      i_Value =i_Y;
      if(m_b_Small)
      {
          i_Value *= 2;
      }

      // reverse the display from top to bottom
      uc_NewValue = m_uca_CorrectionCurve[255 - i_Value];
      p_Pixel = PIXEL_MAKE(uc_NewValue,uc_NewValue,uc_NewValue);
      p_Pixel &= p_ColorMask;

      for(i_X=0;i_X<m_i_ViewSize;i_X ++)
      {
         m_pc_CorrectedBuffer->Set(i_X,i_Y,p_Pixel);
      }
   } // for i_Y

} // CalculateCorrectedBuffer()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::ResetPressed()
{
   if(m_i_CurveMode == CurveModeGamma)
   {
      float f_Y;

      // set gamma to one, and into the middle
      m_f_GammaValue = 1;

      // Calculate y point in the middle (should be 0.5)
      f_Y = pow(0.5,m_f_GammaValue);

      // move curve point to it
      m_pc_GammaCurvePoint->MoveTo(m_i_AreaSize/2.0,
                                   ( m_i_AreaSize) * (1- f_Y ));

      RecalculateCorrectionCurveGamma();
      
      // display also the text value
      SetGammaEditField(m_f_GammaValue);

   }
   else if(m_i_CurveMode == CurveModeLinear)
   {
      int i;
      for(i=0;i<m_i_CurrentCurvePoints;i++)
      {
         m_pc_CurvePoints[i]->SelectObject(false);
         m_pc_CurvePoints[i]->SetVisible(false);
      }
      // no more point
      m_i_CurrentCurvePoints =0;

      // make a single line
      RecalculateLines();
      // draw the middle part
      m_pc_CurveFrame->RedrawAll();

      //Draw the correction curve
      RecalculateCorrectionCurve();
   }

   // signal the change
   CurveChanged(m_uca_CorrectionCurve);

} // ResetPressed()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::AddPressed()
{
   TRACE << "Add pressed\n";

   // no additional points in Gamma mode
   if(m_i_CurveMode == CurveModeGamma)
   {
      return;
   }

   if(m_i_CurrentCurvePoints >= CURVE_MAXIMUM_POINTS)
   {
      // maximum reached, do nothing
      return;
   }

   // now search for the largest distance
   if(m_i_CurrentCurvePoints >=1)
   {
      float f_X;
      float f_Y;
      int   i_LargestDistance;
      int   i_LargestPointIndex;
      int   i_LastPointX;
      int   i;

      i_LastPointX        =0;
      i_LargestPointIndex =0;

      for(i=0;i < m_i_CurrentCurvePoints; i++)
      {
         int   i_X;
         int   i_Difference;

         m_pc_CurvePoints[i]->GetPosition(f_X,f_Y);

         i_X = (int) f_X;

         i_Difference = i_X -i_LastPointX;

         if(i_Difference >i_LargestDistance)
         {
            i_LargestDistance = i_Difference;
            i_LargestPointIndex =i;
         }
         i_LastPointX = i_X;
      } // for i

      if( ( (m_i_AreaSize-1)- i_LastPointX) > i_LargestDistance)
      {
         // largest distance behind the last point
         float f_NewX;
         float f_NewY;

         f_NewX =((m_i_AreaSize-1)+ i_LastPointX) / 2.0;
         m_pc_CurvePoints[m_i_CurrentCurvePoints-1]->GetPosition(f_X,f_Y);
         f_NewY = (0.0 + f_Y) / 2;
         m_pc_CurvePoints[m_i_CurrentCurvePoints]->MoveTo(f_NewX,f_NewY);
         m_pc_CurvePoints[m_i_CurrentCurvePoints]->SetVisible(true);
         m_i_CurrentCurvePoints ++;
      }
      else // largest distance before a valid point
      {
         GraphicObjectPoint *pc_Next;
         float               f_BeforeX;
         float               f_BeforeY;
         float               f_AfterX;
         float               f_AfterY;
         float               f_NewX;
         float               f_NewY;

         if(i_LargestPointIndex ==0)
         {
            f_BeforeX =0.0;
            f_BeforeY =(float) (m_i_AreaSize-1);
         }
         else
         {
           m_pc_CurvePoints[i_LargestPointIndex-1]->GetPosition(f_BeforeX,f_BeforeY);
         }
         m_pc_CurvePoints[i_LargestPointIndex]->GetPosition(f_AfterX,f_AfterY);
         f_NewX = (f_BeforeX + f_AfterX) /2.0;
         f_NewY = (f_BeforeY + f_AfterY) /2.0;

         pc_Next = m_pc_CurvePoints[m_i_CurrentCurvePoints];
         // make space for one in the middle
         for(i=m_i_CurrentCurvePoints;i>i_LargestPointIndex;i--)
         {
            m_pc_CurvePoints[i] = m_pc_CurvePoints[i-1];
         }
         // set values to the next point
         pc_Next->MoveTo(f_NewX,f_NewY);
         pc_Next->SetVisible(true);
         // reintegrate into the middle
         m_pc_CurvePoints[i_LargestPointIndex] = pc_Next;
         m_i_CurrentCurvePoints ++;
      }
   }
   else // no point so far
   {
      // set one in the middle
      m_pc_CurvePoints[0] ->MoveTo(m_i_AreaSize/2.0,
                                   m_i_AreaSize/2.0);
      m_pc_CurvePoints[0] ->SetVisible(true);

      m_i_CurrentCurvePoints =1;
   }

   // adjust the lines according the points
   RecalculateLines();
   RecalculateCorrectionCurve();

   m_pc_CurveFrame->RedrawAll();

   // signal the change
   CurveChanged(m_uca_CorrectionCurve);

} // AddPressed()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::DeletePressed()
{
   GraphicObject *pc_Point;
   int            i_NumberOfSelectedPoints;
   int            i;

   TRACE << "Delete pressed\n";

   // no deletion in Gamma mode
   if(m_i_CurveMode == CurveModeGamma)
   {
      return;
   }

   // look, whether a point is selected

   i_NumberOfSelectedPoints = pc_ObjectLayer->GetNumberOfSelectedObjects(pc_Point);

   if(i_NumberOfSelectedPoints<1)
   {
      return;
   }
   
   pc_Point->SelectObject(false);
   pc_Point->SetVisible(false);

   // now find the point in the array
   for(i=0;i<m_i_CurrentCurvePoints;i++)
   {
      if(pc_Point == m_pc_CurvePoints[i])
      {
         int j;

         for(j= i; j < m_i_CurrentCurvePoints-1 ; j++)
         {
             // move one position to the front
             m_pc_CurvePoints[j] =  m_pc_CurvePoints[j+1];
         }

         // put unused point in the last position
	 m_pc_CurvePoints[m_i_CurrentCurvePoints-1] = 
                                         (GraphicObjectPoint *) pc_Point;
         m_i_CurrentCurvePoints -= 1;
         break;
      }
   } // for i


   RecalculateLines();
   m_pc_CurveFrame->RedrawAll();
   RecalculateCorrectionCurve();

   // signal the change
   CurveChanged(m_uca_CorrectionCurve);

} // DeletePressed()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::LineModeChanged(int i_NewValue)
{
   int i;

   TRACE << "Line Mode changed "
         << i_NewValue
         << "\n";

   m_i_CurveMode = i_NewValue;

   if(m_i_CurveMode == CurveModeLinear)
   {
      // disable the gamma point
      m_pc_GammaCurvePoint->SetVisible(false);

      // enable all points
      for(i=0;i<m_i_CurrentCurvePoints;i++)
      {
         m_pc_CurvePoints[i]->SetVisible(true);
      }

      // clean to main buffer to a grey
      CleanMainBuffer();

      // calculate the lines
      RecalculateLines();

      // redraw the main image
      m_pc_CurveFrame->RedrawAll();

      // calculate the correction curve
      RecalculateCorrectionCurve();

   }
   else if(m_i_CurveMode == CurveModeGamma)
   {

      // disable all points and lines
      for(i=0;i<m_i_CurrentCurvePoints;i++)
      {
         m_pc_CurvePoints[i]->SetVisible(false);
      }

      for(i=0;i<m_i_CurrentCurvePoints+1;i++)
      {
         m_pc_CurveLines[i]->SetVisible(false);
      }

      // enable the gamma point
      m_pc_GammaCurvePoint->SetVisible(true);

      RecalculateCorrectionCurveGamma();
      // redraw the main image
      // m_pc_CurveFrame->RedrawAll();
      SetGammaEditField(m_f_GammaValue);
   }
   SetSensitivity();

   // signal the change
   CurveChanged(m_uca_CorrectionCurve);

} // LineModeChanged()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::SetSensitivity()
{
   if(m_i_CurveMode == CurveModeLinear)
   {
      m_AddButton      ->setEnabled(true);
      m_DeleteButton   ->setEnabled(true);
      m_GammaFactorEdit->setEnabled(false);

   }
   else if(m_i_CurveMode == CurveModeGamma)
   {
      m_AddButton      ->setEnabled(false);
      m_DeleteButton   ->setEnabled(false);
      m_GammaFactorEdit->setEnabled(true);

   }

} // SetSensitivity()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::RecalculateLines()
{
   float f_LastX;
   float f_LastY;
   int   i_CurrentIndex;
   int   i;

   f_LastX =0.0;
   f_LastY =(float) (m_i_AreaSize- 1);

   i_CurrentIndex =0;
   for(i=0;i<m_i_CurrentCurvePoints;i++)
   {
      float f_ActualX;
      float f_ActualY;

      m_pc_CurvePoints[i_CurrentIndex]->GetPosition(f_ActualX,f_ActualY);
      m_pc_CurveLines[i_CurrentIndex]->SetPosition(f_LastX,f_LastY,
                                                   f_ActualX,f_ActualY);
      m_pc_CurveLines[i_CurrentIndex]->SetVisible(true);
      i_CurrentIndex++;

      f_LastX = f_ActualX;
      f_LastY = f_ActualY;
   }

   // set the last line to 255/255
   m_pc_CurveLines[i_CurrentIndex]->SetVisible(true);
   m_pc_CurveLines[i_CurrentIndex]->SetPosition(f_LastX,f_LastY,
                         (float) (m_i_AreaSize-1),0.0);
   i_CurrentIndex++;

   // make lines after invisible
   while(i_CurrentIndex < CURVE_MAXIMUM_POINTS+1)
   {
      m_pc_CurveLines[i_CurrentIndex]->SetVisible(false);
      i_CurrentIndex ++;
   }

} // RecalculateLines()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::GraphicObjektCallback(GraphicObjectMainEvent *pc_ObjectCallbackData)
{
   TRACE <<"Object Callback "
         <<(int) pc_ObjectCallbackData->i_CallbackType
         <<"\n";

   if(pc_ObjectCallbackData->i_CallbackType ==GraphicObjectMainEvent::Moved)
   {
      if(m_i_CurveMode == CurveModeLinear)
      {
         ObjectMovedLinear(pc_ObjectCallbackData);
      }
      else if(m_i_CurveMode == CurveModeGamma)
      {
         ObjectMovedGamma(pc_ObjectCallbackData);
      }

   } // if moved
   else if(  pc_ObjectCallbackData->i_CallbackType
           ==GraphicObjectMainEvent::DragEnd      )
   {
      // signal the change
      CurveChanged(m_uca_CorrectionCurve);
   }

} // GraphicObjektCallback()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::ObjectMovedLinear(GraphicObjectMainEvent *pc_ObjectCallbackData)
{
   int i;

   // find the index of the object
   for(i=0;i<m_i_CurrentCurvePoints;i++)
   {
      if(pc_ObjectCallbackData->pc_Object == m_pc_CurvePoints[i])
      {
         float f_NewX;
         float f_NewY;
         int   j;

         // clip new position into the valid region
         m_pc_CurvePoints[i]->GetPosition(f_NewX,f_NewY);
         if(f_NewX <0.0)
         {
            f_NewX = 0.0;
            m_pc_CurvePoints[i]->MoveTo(f_NewX,f_NewY);
         }
         if(f_NewX >(m_i_AreaSize-1))
         {
            f_NewX = (m_i_AreaSize-1);
            m_pc_CurvePoints[i]->MoveTo(f_NewX,f_NewY);
         }
         if(f_NewY <0.0)
         {
            f_NewY = 0.0;
            m_pc_CurvePoints[i]->MoveTo(f_NewX,f_NewY);
         }
         if(f_NewY >(m_i_AreaSize-1))
         {
            f_NewY = (m_i_AreaSize-1);
            m_pc_CurvePoints[i]->MoveTo(f_NewX,f_NewY);
         }


         // all lower points of the i-point must have lower-equal x values
         for(j=0;j< i;j++)
         {
            float f_OtherX;
            float f_OtherY;
            m_pc_CurvePoints[j]->GetPosition(f_OtherX,f_OtherY);
            if(f_OtherX > f_NewX)
            {
               m_pc_CurvePoints[j]->MoveTo(f_NewX,f_OtherY);
            }
         } // for j - lower

         // all upper points of the i-point must have higher-equal x values
         for(j=i+1;j< m_i_CurrentCurvePoints;j++)
         {
            float f_OtherX;
            float f_OtherY;
            m_pc_CurvePoints[j]->GetPosition(f_OtherX,f_OtherY);
            if(f_OtherX < f_NewX)
            {
               m_pc_CurvePoints[j]->MoveTo(f_NewX,f_OtherY);
            }
         } // for j - lower

         break;
      }
   } // for i


   RecalculateLines();
   RecalculateCorrectionCurve();

   m_pc_CurveFrame->RedrawAll();

} // ObjectMovedLinear()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::ObjectMovedGamma(GraphicObjectMainEvent *pc_ObjectCallbackData)
{
   float f_PositionX;
   float f_PositionY;

   m_pc_GammaCurvePoint->GetPosition(f_PositionX, f_PositionY);

   // invert the Y-value to mathematical system
   f_PositionY = (m_i_AreaSize-1) - f_PositionY;

   // convert to a range 0 .. 1.0

   f_PositionX /= (m_i_AreaSize-1);
   if(f_PositionX > 0.99)
   {
      f_PositionX = 0.99;
   }
   if(f_PositionX <= 0.0)
   {
      f_PositionX = 1.0 / (m_i_AreaSize-1);
   }

   f_PositionY /= (m_i_AreaSize-1);
   if(f_PositionY > 0.99)
   {
      f_PositionY = 0.99;
   }
   if(f_PositionY <= 0.0)
   {
      f_PositionY = 1.0 / (m_i_AreaSize-1);
   }

   // calculage a gamma value out of the 2 koordinates
   m_f_GammaValue = log(f_PositionY) / log(f_PositionX);

   RecalculateCorrectionCurveGamma();

   SetGammaEditField(m_f_GammaValue);

} // ObjectMovedGamma()

//===========================================================================
//
//===========================================================================

void *QUCorrectionCurve::GraphicObjectCallbackCB(void *thisData,
                                                void *GraphicObjectCallbackData)
{
   QUCorrectionCurve *pc_This;

   pc_This = (QUCorrectionCurve *) thisData;

   pc_This->GraphicObjektCallback((GraphicObjectMainEvent *) GraphicObjectCallbackData);

   return NULL;
} // GraphicObjectCallbackCB()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::RecalculateCorrectionCurve()
{
   int i;

   if(m_i_CurveMode == CurveModeGamma)
   {
   }
   else if (m_i_CurveMode == CurveModeLinear)
   {
      if(m_i_CurrentCurvePoints==0)
      {
         // no point, make 1 to 1 curve
         for(i=0;i<256;i++)
         {
            m_uca_CorrectionCurve[i] = i;
         }

      }
      else // one or more points
      {
         float f_LastX;
         float f_LastY;
         float f_ActualX;
         float f_ActualY;
         int   i_Curve;

         f_LastX = 0;
         f_LastY = 255.0;

         for(i_Curve=0;i_Curve<m_i_CurrentCurvePoints;i_Curve++)
         {
            m_pc_CurvePoints[i_Curve]->GetPosition(f_ActualX,f_ActualY);
            if(m_b_Small)
            {
               f_ActualX *= 2;
               f_ActualY *= 2;
            }
            RecalculateCorrectionCurveLine(f_LastX,f_LastY,
                                           f_ActualX,f_ActualY);

            f_LastX = f_ActualX;
            f_LastY = f_ActualY;
         } // for i;

         if(f_LastX <255.0)
         {
            // now draw the last line to the end
            RecalculateCorrectionCurveLine(f_LastX,f_LastY,
                                     255.0,0.0);
         }

      } // else >=1 points
   } // Linear

   // now redisplay the curve on the display
   CalculateCorrectedBuffer();

   m_pc_CorrectedFrame->RedrawAll();

} // RecalculateCorrectionCurve()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::RecalculateCorrectionCurveLine(float f_LastX,
                                       float f_LastY,
                                       float f_ActualX,
                                       float f_ActualY)
{
   float f_DiffX;
   float f_DiffY;
   int   i_RunXStart;
   int   i_RunXEnd;
   int   i_RunX;


   i_RunXStart = (int) (f_LastX +0.2);
   i_RunXEnd   = (int) (f_ActualX +0.2);

   f_DiffX = (f_ActualX - f_LastX);
   f_DiffY = (f_ActualY - f_LastY);

   for(i_RunX=i_RunXStart; i_RunX <= i_RunXEnd;i_RunX ++)
   {
       float f_Faktor;
       int   i_ValueY;

       f_Faktor = ((float)(i_RunX-i_RunXStart)) / (f_DiffX);

       i_ValueY = (int)(f_LastY + f_Faktor*f_DiffY);
       if(i_ValueY >255)
       {
          i_ValueY = 255;
       }
       else if(i_ValueY <0)
       {
          i_ValueY =0;
       }

       m_uca_CorrectionCurve[i_RunX] = 255 - (unsigned char) (i_ValueY);
   } // for i_RunX


} // RecalculateCorrectionCurveLine()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::CleanMainBuffer()
{
   int i_X;
   int i_Y;

   for(i_Y=0; i_Y < m_i_AreaSize; i_Y ++)
   {
      for(i_X=0; i_X < m_i_AreaSize; i_X ++)
      {
         // set it grey
         m_pc_CurveBuffer->SetRaw(i_X,i_Y,0xcccccc);
      }
   }
} // CleanMainBuffer()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::RecalculateCorrectionCurveGamma()
{
   int i;
   CleanMainBuffer();

   // calculate the complete curve
   for(i=0;i<256;i++)
   {
      float f_X;
      float f_Y;
      int   i_Y;

      f_X = (float) i / (float) 255;

      // calculate the power
      f_Y = pow(f_X, m_f_GammaValue);

      // range 0 .. m_i_AreaSize-1)
      f_Y *= 255.0;

      // make int
      i_Y = (int)( f_Y +0.5);

      // clip
      if(i_Y < 0)
      {
         i_Y = 0;
      }
      else if(i_Y > 255)
      {
         i_Y = 255;
      }

      m_uca_CorrectionCurve[i] = i_Y;
   }

   // put the pixel into the graphics

   for(i=0;i<(m_i_AreaSize);i++)
   {
      int i_Value;
      int i_Height;
      i_Value = i;
      if(m_b_Small)
      {
         i_Value *= 2;
      }

      i_Height = m_uca_CorrectionCurve[i_Value];
      if(m_b_Small)
      {
         i_Height /=2 ;
      }
      m_pc_CurveBuffer->SetRaw(i,(m_i_AreaSize-1)-i_Height,0);
   } // for i



   // now redisplay the curve on the display
   CalculateCorrectedBuffer();

   m_pc_CorrectedFrame->RedrawAll();
   m_pc_CurveFrame->RedrawAll();

} // RecalculateCorrectionCurveGamma()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::TextChanged(const QString &qs_NewText)
{
   QString qs_Text;
   float   f_NewFactor;
   float   f_Y;

   if(m_b_SelfUpdate)
   {
      return;
   }

   // get the actual value in the width edit
   qs_Text = m_GammaFactorEdit->text();

   f_NewFactor = 1.0;
   sscanf((const char *) qs_Text,"%f",&f_NewFactor);

   if(f_NewFactor> 9.0)
   {
      f_NewFactor = 9.0;
   }
   else if(f_NewFactor < 0.1)
   {
      f_NewFactor = 0.1;
   }
   m_f_GammaValue = f_NewFactor;


   f_Y = pow(0.5,m_f_GammaValue);

   m_pc_GammaCurvePoint->MoveTo(m_i_AreaSize / 2 ,
                                (m_i_AreaSize-1) * (1- f_Y ));

   RecalculateCorrectionCurveGamma();

   // signal the change
   CurveChanged(m_uca_CorrectionCurve);


} // TextChanged()

//===========================================================================
//
//===========================================================================

void QUCorrectionCurve::SetGammaEditField(float f_Factor)
{
   QString qs_String;

   qs_String.sprintf("%8f",f_Factor);

   m_b_SelfUpdate = true;
   m_GammaFactorEdit->setText(qs_String);
   m_b_SelfUpdate = false;
} // SetGammaEditField()

//===========================================================================
//
//===========================================================================


