
#include "RedEyeReduction.h"
#include "ColorCalculation.h"


//============================================================================
//
//============================================================================

RedEyeReduction::RedEyeReduction()
{
   m_pc_Layer         = NULL;
   m_pc_WorkImage     = NULL;
   m_pc_SelectionMask = NULL;

} // RedEyeReduction() constructor

//============================================================================
//
//============================================================================

RedEyeReduction::~RedEyeReduction()
{
   if(m_pc_SelectionMask != NULL)
   {
      delete m_pc_SelectionMask;
      m_pc_SelectionMask = NULL;
   }
   
} // ~RedEyeReduction()

//============================================================================
//
//============================================================================

void RedEyeReduction::SetLayer(GraphicObjectLayer *pc_Layer)
{
   m_pc_Layer = pc_Layer;
} // SetLayer()

//============================================================================
//
//============================================================================

void RedEyeReduction::SetImage(PixelBuffer *pc_Image)
{
   m_pc_WorkImage = pc_Image;
} // SetImage()

//============================================================================
//
//============================================================================

void RedEyeReduction::SetDarkeningFactor(int i_Factor)
{
   m_i_DarkeningFactor = i_Factor;

} // SetDarkeningFactor()

//============================================================================
//
//============================================================================

void RedEyeReduction::Reduce(RectRegion &c_ChangedRegion)
{
   BufferPainter c_Painter;
   
   int i_NumberOfEllipses;
   int i;

   int i_RunX;
   int i_RunY;
   int i_Width;
   int i_Height;
   
   // nothing changed
   c_ChangedRegion.SetEmpty();
   
   if(   m_pc_WorkImage == NULL
      || m_pc_Layer     == NULL)
   {
      return;
   }

   // get number of ellipses
   i_NumberOfEllipses = m_pc_Layer->GetObjectCount();
   // must at least one there
   if( i_NumberOfEllipses == 0)
   {
      return ;
   }
   
   // delete an old selection mask -- if existing
   if(m_pc_SelectionMask != NULL)
   {
      delete m_pc_SelectionMask;
      m_pc_SelectionMask = NULL;
   }

   // alloc mask - same size as work image, 1 bit depth
   m_pc_SelectionMask = new PixelBuffer(m_pc_WorkImage,1);
   if(m_pc_SelectionMask == NULL)
   {
      return;
   }

   // set the painter to draw with 1
   c_Painter.SetBorderColor(1);
   c_Painter.SetInnerColor(1);
   c_Painter.SetDrawInnerEnable(true);
   
   // now draw the ellipses into the mask
   for( i=0; i < i_NumberOfEllipses; i++)
   {
      GraphicObject *pc_Object;

      // get the current object
      pc_Object = m_pc_Layer->GetObject(i);

      // is the object really an ellipse?
      if(pc_Object->GetObjectType() == GraphicObject::TypeEllipse)
      {
         GraphicObjectEllipse *pc_Ellipse;
         RectRegion            c_EllipseSize;
         int                   i_CenterX;
         int                   i_CenterY;
         int                   i_RadiusX;
         int                   i_RadiusY; 

         pc_Ellipse = (GraphicObjectEllipse *) pc_Object;

         c_EllipseSize = pc_Ellipse->GetSize();

         // calculate center adn radius of the ellipse
         i_CenterX = ( c_EllipseSize.GetStartX() + c_EllipseSize.GetEndX() )
                     / 2;
         i_CenterY = ( c_EllipseSize.GetStartY() + c_EllipseSize.GetEndY() )
                     / 2;
         i_RadiusX = i_CenterX - c_EllipseSize.GetStartX();
         i_RadiusY = i_CenterY - c_EllipseSize.GetStartY();
         if(    i_RadiusX>0
             && i_RadiusY >0)
         {
            // draw ellipse into the selection mask
            c_Painter.DrawEllipse( m_pc_SelectionMask,
                                   i_CenterX,
                                   i_CenterY,
                                   i_RadiusX,
                                   i_RadiusY);
         } // if radius greater
         
      } // if ellipse
   } // for all ellipses

   // in the selection mask every pixel of all ellipses are masked
   i_Width  = m_pc_WorkImage->GetWidth();
   i_Height = m_pc_WorkImage->GetHeight();

   for(i_RunY=0; i_RunY<i_Height; i_RunY++)
   {
      for(i_RunX=0; i_RunX<i_Width; i_RunX++)
      {
         Pixel p_Mask;
         p_Mask = m_pc_SelectionMask->Get(i_RunX,i_RunY);
         if(p_Mask)
         {
            bool b_Changed;
            b_Changed = CheckPoint(i_RunX, i_RunY);
            if(b_Changed)
            {
               c_ChangedRegion.AddPoint(i_RunX,i_RunY);
            }
         } // if mask set
         
      }   // for run_X
   } // for run y

   
} // Reduce()

//============================================================================
//
//============================================================================
bool  RedEyeReduction::CheckPoint(int i_X, int i_Y)
{
   Pixel p_Current;
   bool  b_Changed ;
   int i_Red;
   int i_Blue;
   int i_Green;
   int i_Hue;
   int i_Lightness;
   int i_Saturation;
   

   b_Changed = false;
   p_Current = m_pc_WorkImage->Get(i_X,i_Y);

   i_Red   = PIXEL_RED  (p_Current);
   i_Green = PIXEL_GREEN(p_Current);
   i_Blue  = PIXEL_BLUE (p_Current);

   ColorCalculation::RGBToHLS(i_Red,i_Green,i_Blue,
                              i_Hue,i_Lightness,i_Saturation);

   if(   i_Hue<30
      || i_Hue>330)
   {
      if(i_Saturation>30 )
      {
         Pixel p_New;
         int i_NewLightness;
         int i_LightnessHelp;
         int i_NewSaturation;
         int i_DampingFactor;

         // damping factor
         i_DampingFactor = 20 + ((100-i_Saturation));
         // damping factor now between 20 and 90

         i_LightnessHelp = i_Lightness * i_DampingFactor/100;

         i_NewLightness = (i_Lightness + i_LightnessHelp*m_i_DarkeningFactor)
                          / (m_i_DarkeningFactor +1);
         
         
         i_NewSaturation = 20;

         ColorCalculation::HLSToRGB(
                              i_Hue,i_NewLightness,i_NewSaturation,
                              i_Red,i_Green,i_Blue);
         p_New = PIXEL_MAKE(i_Red,i_Green,i_Blue);

         m_pc_WorkImage->Set(i_X,i_Y,p_New);
         
         b_Changed = true;
      }
   }
   return b_Changed;
   
} // CheckPoint()

//============================================================================
//
//============================================================================
